#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import (division, print_function, absolute_import,
                        unicode_literals)

__all__ = ["Dataset"]

import copy
from . import kplr

# These dependencies are optional for the ``kplr`` module but required to load
# a dataset. An ``ImportError`` exception will be raised when you try to load
# a file if any of these aren't installed.
try:
    import numpy as np
except ImportError:
    np = None

try:
    import pyfits
except ImportError:
    pyfits = None

# ``untrendy`` is only required if you try to "untrend" a dataset so the
# ``ImportError`` will be raised then.
try:
    import untrendy
except ImportError:
    untrendy = None


class Dataset(object):
    """
    An interface to a Kepler light curve dataset.

    The object parses out various useful parameters:

    * ``texp``: the exposure time in days
    * ``time``: the array of timestamps in KBJD
    * ``bjd``: the same array in BJD
    * ``sapflux``, ``sapferr``, ``sapivar``: the raw aperture photometry
    * ``pdcflux``, ``pdcferr``, ``pdcivar``: the PDC corrected light curve
    * ``flux``, ``ferr``, ``ivar``: the "untrended" and normalized light curve

    :param filename:
        The filename of the FITS file in the standard Kepler format.

    :param untrend: (optional)
        Should the light curve be "untrended" using ``untrendy``?

    :param **untrendy_args: (optional)
        Any additional keyword arguments that should be passed to
        ``untrendy.untrend()``.

    """

    __type__ = "lc"
    vector = []

    def __init__(self, filename, untrend=False, **untrendy_args):
        if not np or not pyfits:
            raise ImportError("numpy and pyfits are required to load a "
                              "dataset.")

        self.filenames = [filename]
        with pyfits.open(filename) as f:
            # Find the cadence of the dataset and save the approximate
            # exposure time.
            self.cadence = (0 if f[0].header["OBSMODE"] == "short cadence"
                            else 1)
            self.texp = kplr.EXPOSURE_TIMES[self.cadence]

            # Convert the exposure time to days so that it is in consistent
            # units.
            self.texp /= 1440.

            # Load the actual data.
            data = f[1].data

            # Read in the timestamps in KBJD.
            self.time = np.array(data["TIME"], dtype=float)

            # Check quality flags.
            self.quality = np.array(data["SAP_QUALITY"], dtype=int)
            self.qualitymask = ~np.array(self.quality, dtype=bool)

            # Read in the raw aperture photometry of the light curve.
            self.sapflux = np.array(data["SAP_FLUX"], dtype=float)
            self.sapferr = np.array(data["SAP_FLUX_ERR"], dtype=float)
            self.sapmask = ~(np.isnan(self.time) +
                             np.isnan(self.sapflux) +
                             np.isnan(self.sapferr))
            self.sapivar = np.zeros_like(self.sapferr)
            self.sapivar[self.sapmask] = 1.0 / self.sapferr[self.sapmask] ** 2

            # Read in the PDC corrected light curve.
            self.pdcflux = np.array(data["PDCSAP_FLUX"], dtype=float)
            self.pdcferr = np.array(data["PDCSAP_FLUX_ERR"], dtype=float)
            self.pdcmask = ~(np.isnan(self.time) +
                             np.isnan(self.pdcflux) +
                             np.isnan(self.pdcferr))
            self.pdcivar = np.zeros_like(self.pdcferr)
            self.pdcivar[self.pdcmask] = 1.0 / self.pdcferr[self.pdcmask] ** 2

            # Compute the BJD.
            bjdconv = f[1].header["BJDREFI"] + f[1].header["BJDREFF"]
            self.bjd = self.time + bjdconv

        if untrend:
            if not untrendy:
                raise ImportError("The untrendy module is required for "
                                  "'untrending' of light curves.")

            self.mask = self.sapmask[:]
            m = self.sapmask * self.qualitymask
            self.flux = np.zeros_like(self.sapflux)
            self.ferr = np.zeros_like(self.sapflux)

            # Untrend the light curve using untrendy.
            mu = np.median(self.sapflux[m])
            untrendy_args["fill_times"] = untrendy_args.get("fill_times",
                                                            10 ** -1.25)
            model = untrendy.fit_trend(self.time[m], self.sapflux[m] / mu,
                                       self.sapferr[m] / mu, **untrendy_args)
            factor = mu * model(self.time[self.mask])
            self.flux[self.mask] = self.sapflux[self.mask] / factor
            self.ferr[self.mask] = self.sapferr[self.mask] / factor

            self.ivar = np.zeros_like(self.ferr)
            self.ivar[self.mask] = 1.0 / self.ferr[self.mask] ** 2

        else:
            self.flux = self.pdcflux[:]
            self.ferr = self.pdcferr[:]
            self.ivar = self.pdcivar[:]
            self.mask = self.pdcmask[:]

            factor = np.median(self.pdcflux[self.pdcmask])
            self.flux[self.mask] /= factor
            self.ferr[self.mask] /= factor
            self.ivar[self.mask] *= factor * factor

    def __add__(self, other):
        if self.texp != other.texp:
            raise ValueError("Incompatible exposure times.")

        new = copy.copy(self)
        new.filenames += other.filenames

        new.time = np.append(self.time[self.mask], other.time[other.mask])
        inds = np.argsort(new.time)
        new.time = new.time[inds]

        for attr in ["sapflux", "sapflux", "sapivar", "sapmask",
                     "pdcflux", "pdcferr", "pdcivar", "pdcmask",
                     "flux", "ferr", "ivar", "mask", "bjd"]:
            setattr(new, attr, np.append(getattr(self, attr)[self.mask],
                    getattr(other, attr)[other.mask])[inds])

        return new

    def __len__(self):
        return 0



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import (division, print_function, absolute_import,
                        unicode_literals)

__all__ = ["get_quad_coeffs", "LDCoeffAdaptor"]

import os
import requests
import numpy as np

from .config import KPLR_ROOT


def get_quad_coeffs(teff=5778, logg=None, feh=None, model="sing09",
                    data_root=None):
    """
    Get the quadratic coefficients for the standard Kepler limb-darkening
    profile.

    :param teff: (optional)
        The effective temperature in degrees K.

    :param logg: (optional)
        The log10 surface gravity in cm/s/s.

    :param feh: (optional)
        The metallicity [Fe/H].

    :model: (optional)
        The theoretical model to use. See :class:`LDCoeffAdaptor`.

    :param data_root: (optional)
        The local base directory where the grids will be downloaded to. This
        can also be set using the ``KPLR_ROOT`` environment variable. The
        default value is ``~/.kplr``.

    """
    # Find the LD coefficients from the theoretical models.
    a = LDCoeffAdaptor(model=model)
    return a.get_coeffs(teff, logg=logg, feh=feh)


class LDCoeffAdaptor(object):
    """
    Wrapper around various theoretical models for the coefficients of limb
    darkening profiles for Kepler stars.

    :param model: (optional)
        The name of the model that you would like to use. The currently
        supported models are ``sing09`` and ``claret11``. When you use a model
        for the first time, it will download the data file and save it to
        ``{data_root}/ldcoeffs/claret11.txt``.

    :param data_root: (optional)
        The local base directory where the grids will be downloaded to. This
        can also be set using the ``KPLR_ROOT`` environment variable. The
        default value is ``~/.kplr``.

    """

    models = {
        "sing09": "http://broiler.astrometry.net/~dfm265/ldcoeffs/sing.txt",
        "claret11": "http://broiler.astrometry.net/~dfm265/ldcoeffs/claret.txt"
    }

    def __init__(self, model="sing09", data_root=None):
        if model not in self.models:
            raise TypeError(("Unrecognized model '{0}'. The allowed values "
                             "are {1}.").format(model, self.allowed_models))

        # Save the provided data root directory and fall back on the
        # ``KPLR_ROOT`` environment variable.
        self.data_root = data_root
        if data_root is None:
            self.data_root = KPLR_ROOT

        # Download the data table if it doesn't always exist.
        local_fn = os.path.join(KPLR_ROOT, "ldcoeffs",
                                "{0}.txt".format(model))
        if not os.path.exists(local_fn):
            print("Downloading the data file for the '{0}' model"
                  .format(model))
            r = requests.get(self.models[model])
            if r.status_code != requests.codes.ok:
                r.raise_for_status()
            try:
                os.makedirs(os.path.dirname(local_fn))
            except os.error:
                pass
            open(local_fn, "w").write(r.content)
            print("  .. Finished.")

        if model == "sing09":
            data = np.loadtxt(local_fn, skiprows=10, usecols=range(6))
            self.T = data[:, 0]
            self.logg = data[:, 1]
            self.feh = data[:, 2]
            self.mu1 = data[:, 4]
            self.mu2 = data[:, 5]

        else:
            data = np.loadtxt(local_fn, skiprows=59, delimiter="|",
                              usecols=range(7))
            self.T = data[:, 2]
            self.logg = data[:, 1]
            self.feh = data[:, 3]
            self.mu1 = data[:, 5]
            self.mu2 = data[:, 6]

    def get_coeffs(self, teff, logg=None, feh=None):
        """
        Get the coefficients of the quadratic limb darkening profile given by
        theory.

        :param teff:
            The effective temperature in degrees K.

        :param logg: (optional)
            The log10 surface gravity in cm/s/s.

        :param feh: (optional)
            The metallicity [Fe/H].

        """
        T0 = self.T[np.argmin(np.abs(self.T - teff))]
        inds = self.T == T0
        if logg is not None:
            lg0 = self.logg[np.argmin(np.abs(self.logg - logg))]
            inds *= self.logg == lg0
        if feh is not None:
            feh0 = self.feh[np.argmin(np.abs(self.feh - feh))]
            inds *= self.feh == feh0
        return np.mean(self.mu1[inds]), np.mean(self.mu2[inds])
